@@ -1,13 +1,15 @@
+import json
+
+import requests
+
 from cli.utils import download_url, fetch_auth, parse_filter
 from zpy.files import to_pathlib_path
-import json
-import requests
 
 DATASET_TYPES = ["uploaded-data-sets", "generated-data-sets", "job-data-sets"]
 
 
 @fetch_auth
-def filter_datasets(dfilter, url, auth_headers):
+def filter_datasets(dfilter, project, url, auth_headers):
     """filter datasets
 
     Filter dataset objects on ZumoLabs backend by given dfilter.
@@ -16,6 +18,7 @@ def filter_datasets(dfilter, url, auth_headers):
 
     Args:
         dfilter (str): filter query for datasets
+        project (str): project uuid
         url (str): backend endpoint
         auth_headers: authentication for backend
 
@@ -25,9 +28,14 @@ def filter_datasets(dfilter, url, auth_headers):
     filtered_datasets = {}
     field, pattern, regex = parse_filter(dfilter)
     for dataset_type in DATASET_TYPES:
-        endpoint = f"{url}/api/v1/{dataset_type}/?{field}__{pattern}={regex}"
+        endpoint = f"{url}/api/v1/{dataset_type}/"
+        params = {
+            **params,
+            f"{field}__{pattern}": regex,
+        }
+
         while endpoint is not None:
-            r = requests.get(endpoint, headers=auth_headers)
+            r = requests.get(endpoint, params=params, headers=auth_headers)
             if r.status_code != 200:
                 r.raise_for_status()
             response = json.loads(r.text)
@@ -38,7 +46,7 @@ def filter_datasets(dfilter, url, auth_headers):
 
 
 @fetch_auth
-def create_generated_dataset(name, sim_name, config, url, auth_headers):
+def create_generated_dataset(name, sim_name, config, project, url, auth_headers):
     """create dataset
 
     Create generated dataset on ZumoLabs backend which will launch
@@ -48,6 +56,7 @@ def create_generated_dataset(name, sim_name, config, url, auth_headers):
         name (str): name of dataset
         sim_name (str): name of sim the dataset is built from
         config (dict): configration of sim for this dataset
+        project (str): project uuid
         url (str): backend endpoint
         auth_headers: authentication for backend
     """
@@ -62,7 +71,12 @@ def create_generated_dataset(name, sim_name, config, url, auth_headers):
     endpoint = f"{url}/api/v1/generated-data-sets/"
     r = requests.post(
         endpoint,
-        data={"sim": sim["id"], "config": json.dumps(config), "name": name},
+        data={
+            "project": project,
+            "sim": sim["id"],
+            "config": json.dumps(config),
+            "name": name,
+        },
         headers=auth_headers,
     )
     if r.status_code != 201:
@@ -70,7 +84,7 @@ def create_generated_dataset(name, sim_name, config, url, auth_headers):
 
 
 @fetch_auth
-def create_uploaded_dataset(name, path, url, auth_headers):
+def create_uploaded_dataset(name, path, project, url, auth_headers):
     """upload dataset
 
     Upload dataset to S3 through ZumoLabs backend and
@@ -79,13 +93,14 @@ def create_uploaded_dataset(name, path, url, auth_headers):
     Args:
         name (str): name of dataset
         path (str): path to dataset
+        project (str): project uuid
         url (str): backend endpoint
         auth_headers: authentication for backend
     """
     endpoint = f"{url}/api/v1/uploaded-data-sets/"
     r = requests.post(
         endpoint,
-        data={"name": name},
+        data={"name": name, "project": project},
         files={"file": open(path, "rb")},
         headers=auth_headers,
     )
@@ -130,12 +145,14 @@ def download_dataset(name, path, dataset_type, url, auth_headers):
     return output_path
 
 
-def fetch_datasets():
+def fetch_datasets(filters):
     """fetch datasets
 
     Fetch dataset objects from ZumoLabs backend. Fetches all three
     datasets types uploaded, generated, and job.
 
+    Args:
+        filters (dict): query param filters for API call
     Returns:
         list: paginated sorted datasets for all types
     """
@@ -144,17 +161,18 @@ def fetch_datasets():
         short_dataset_type = dataset_type.split("-")[0]
         datasets += [
             (lambda d: d.update({"type": short_dataset_type}) or d)(d)
-            for d in _fetch_type_datasets(dataset_type)
+            for d in _fetch_type_datasets(dataset_type, filters)
         ]
     return sorted(datasets, key=lambda i: i["created_at"], reverse=True)
 
 
 @fetch_auth
-def _fetch_type_datasets(dataset_type, url, auth_headers):
+def _fetch_type_datasets(dataset_type, filters, url, auth_headers):
     """fetch type of datasets
 
     Args:
         dataset_type (str): type of dataset to fetch
+        filters (dict): query param filters for API call
         url (str): backend endpoint
         auth_headers: authentication for backend
 
@@ -162,7 +180,7 @@ def _fetch_type_datasets(dataset_type, url, auth_headers):
         list: paginated datasets for given type
     """
     endpoint = f"{url}/api/v1/{dataset_type}/"
-    r = requests.get(endpoint, headers=auth_headers)
+    r = requests.get(endpoint, headers=auth_headers, params=filters)
     if r.status_code != 200:
         r.raise_for_status()
     return json.loads(r.text)["results"]
