@@ -4,7 +4,7 @@ import json
 
 
 @fetch_auth
-def create_new_job(name, operation, config, datasets, url, auth_headers):
+def create_new_job(name, operation, config, datasets, project, url, auth_headers):
     """create job
 
     Create a job object on ZumoLabs backend. This will trigger the backend
@@ -17,9 +17,11 @@ def create_new_job(name, operation, config, datasets, url, auth_headers):
         datasets (dict): list of dataset ids
         url (str): backend endpoint
         auth_headers: authentication for backend
+        project (str): project uuid
     """
     endpoint = f"{url}/api/v1/jobs/"
     data = {
+        "project": project,
         "operation": operation.upper(),
         "name": name,
         "input_data_sets": datasets,
@@ -31,12 +33,13 @@ def create_new_job(name, operation, config, datasets, url, auth_headers):
 
 
 @fetch_auth
-def fetch_jobs(url, auth_headers):
+def fetch_jobs(filters, url, auth_headers):
     """fetch jobs
 
     Fetch job objects from ZumoLabs backend.
 
     Args:
+        filters (dict): query param filters for API call
         url (str): backend endpoint
         auth_headers: authentication for backend
 
@@ -44,7 +47,7 @@ def fetch_jobs(url, auth_headers):
         list: list of jobs
     """
     endpoint = f"{url}/api/v1/jobs/"
-    r = requests.get(endpoint, headers=auth_headers)
+    r = requests.get(endpoint, headers=auth_headers, params=filters)
     if r.status_code != 200:
         r.raise_for_status()
     return json.loads(r.text)["results"]
